/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/**
 * @file Program.h
 *
 * @date    2012-07-30
 * @author  Philipp Kegel
 */

#ifndef PROGRAM_H_
#define PROGRAM_H_

#include <cstddef>
#include <vector>

#include <boost/shared_ptr.hpp>

#define __CL_ENABLE_EXCEPTIONS
#include <CL/cl.hpp>

#include <dcl/DCLTypes.h>
#include <dcl/Kernel.h>
#include <dcl/Program.h>
#include <dcl/ProgramBuildListener.h>
#include <dcl/Remote.h>

namespace dcld {

/* forward declaration */
class Context;

/******************************************************************************/

/**
 * @brief A decorator for a native program.
 *
 * This wrapper is required to receive program source or binaries from a host
 * and for notifying program build listeners about completed programs builds.
 */
class Program :
        public dcl::Program { // implement
public:
    Program(
            const boost::shared_ptr<Context>&   /* context */,
            size_t                              /* length */);
    Program(
            const boost::shared_ptr<Context>&   /* context */,
            const VECTOR_CLASS<cl::Device>&     /* devices */,
            const std::vector<size_t>&          /* lengths */,
            VECTOR_CLASS<cl_int> *              /* binary_status */);
    virtual ~Program();

    operator cl::Program() const;

#if 0
    void build(
            const std::vector<cl::Device>&                      /* devices */,
            const char *                                        /* options */,
            const boost::shared_ptr<dcl::ProgramBuildListener>& /* programBuildListener */);
#endif
    void build(
            const std::vector<cl::Device>&  /* devices */,
            const char *                    /* options */,
            dcl::dcl_object_id              /* programBuildId */);

    void createKernels(
            std::vector<boost::shared_ptr<dcl::Kernel> >&   /* kernels */);

    const boost::shared_ptr<Context>& context() const;

private:
    /* Programs must not be copied */
    Program(const Program&);
    Program& operator=(const Program&);

    boost::shared_ptr<Context> _context; /**< Context associated with program */

    cl::Program _program; /**< Native program */
};

} /* namespace dcld */

#endif /* PROGRAM_H_ */
