/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/*
 * dOpenCLd.h
 *
 *  Created on: 16.01.2011
 *      Author: Philipp Kegel
 *
 * dOpenCL daemon class
 */

#ifndef DOPENCLD_H_
#define DOPENCLD_H_

#include <map>

#include <boost/thread/condition_variable.hpp>
#include <boost/thread/mutex.hpp>

#include <dcl/ComputeNode.h>
#include <dcl/ConnectionListener.h>
#include <dcl/ConnectionManager.h>
#include <dcl/Host.h>

#include "DeviceManager.h"

namespace dcld {

/* forward declarations */
class Session;

/******************************************************************************/

class dOpenCLd: public dcl::ConnectionListener {
public:
	dOpenCLd(
			const std::string&  /* url */,
			const std::string *platform = 0);
	virtual ~dOpenCLd();

	/**
	 * @brief Does what the daemon does.
	 *
	 * This is a blocking method. Use terminate to return from this call.
	 */
	void run();
	/**
	 * @brief Terminates the daemon.
	 *
	 * This method signals the daemon to stop gracefully.
	 */
	void terminate();

	/*
	 * dOpenCL connection listener API
	 */
	bool connected(dcl::Host&);
	void disconnected(dcl::Host&);
	bool connected(dcl::ComputeNode&);
	void disconnected(dcl::ComputeNode&);

private:
	/* Daemon must not be copied */
	dOpenCLd(const dOpenCLd&);
	dOpenCLd& operator=(const dOpenCLd&);

	dcl::ComputeNodeConnectionManager *_connectionManager;

	DeviceManager _deviceManager;

	bool _interrupt;
	boost::mutex _interruptMutex;
	boost::condition_variable_any _interrupted;

	std::map<const dcl::Host *, Session *> _sessions; /**< Sessions */
	mutable boost::mutex _sessionsMutex;
};

} /* namespace dcld */

#endif /* DOPENCLD_H_ */
