/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/*
 * ContextMessage.h
 *
 *  Created on: 27.03.2012
 *      Author: Sebastian Pribnow
 */

#ifndef CONTEXTMESSAGE_H_
#define CONTEXTMESSAGE_H_

#include <string>

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

#include <gcf/GCFTypedefs.h>
#include <gcf/Serializable.h>

#include <dcl/Binary.h>
#include <dcl/DCLTypes.h>

namespace dcl {

namespace comm {

/**
 * @brief Notification of context error.
 *
 * This message is sent from compute nodes to the host.
 */
class ContextErrorMessage: public gcf::Serializable {
public:
	ContextErrorMessage(
			dcl_object_id       /* contextId */,
			const std::string&  /* errorInfo */,
			const dcl::Binary&	/* privateInfo */);
	ContextErrorMessage(const ContextErrorMessage&);
	virtual ~ContextErrorMessage();

	dcl_object_id contextId(void) const;
	const std::string& errorInfo(void) const;
	const dcl::Binary& privateInfo(void) const;

	DECLARE_SERIALIZABLE_PUBLIC(ContextErrorMessage, 8599)

private:
	dcl_object_id _contextId;
	std::string _errorInfo;
	dcl::Binary _privateInfo;

	DECLARE_SERIALIZABLE_EVENT_PRIVATE(ContextErrorMessage)
};

/******************************************************************************/

/**
 * @brief Notification of call to printf.
 *
 * This message is sent from compute nodes to the host.
 */
class ContextPrintfMessage: public gcf::Serializable {
public:
    ContextPrintfMessage(
            dcl_object_id       /* contextId */,
            const std::string&  /* printfData */);
    ContextPrintfMessage(const ContextPrintfMessage&);
    virtual ~ContextPrintfMessage();

    dcl_object_id contextId(void) const;
    const std::string& printfData(void) const;

    DECLARE_SERIALIZABLE_PUBLIC(ContextPrintfMessage, 8598)

private:
    dcl_object_id _contextId;
    std::string _printfData;

    DECLARE_SERIALIZABLE_EVENT_PRIVATE(ContextPrintfMessage)
};

} /* namespace comm */

} /* namespace dcl */

#endif /* CONTEXTMESSAGE_H_ */
