/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/**
 * @file    CreateBuffer.h
 *
 * @date    17.06.2011
 * @author  Marius Holzwarth
 * @author  Tim Humernbrum
 */

#ifndef CREATEBUFFER_H_
#define CREATEBUFFER_H_

#include <cstddef>

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

#include <gcf/GCFTypedefs.h>

#include <dcl/DCLTypes.h>

#include "Request.h"

namespace dcl {

namespace comm {

class CreateBuffer : public Request {
public:
	CreateBuffer(
			dcl_object_id bufferId,
			dcl_object_id contextId,
			cl_mem_flags flags,
			size_t size);
	CreateBuffer(const CreateBuffer&);

	dcl_object_id bufferId(void) const;
	dcl_object_id contextId(void) const;
	cl_mem_flags flags(void) const;
	size_t size(void) const;

	DECLARE_SERIALIZABLE_PUBLIC(CreateBuffer, 100 + CREATE_BUFFER)

private:
	dcl_object_id _bufferId;
	dcl_object_id _contextId;
	gcf_uint64 _flags;
	gcf_uint32 _size;

	DECLARE_SERIALIZABLE_EVENT_PRIVATE(CreateBuffer)
};

} /* namespace comm */

} /* namespace dcl */

#endif /* CREATEBUFFER_H_ */
