/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/**
 * @file    CreateCommandQueue.h
 *
 * @date    May 31, 2011
 * @author  Tunc Taylan Turunc
 */

#ifndef CREATECOMMANDQUEUE_H_
#define CREATECOMMANDQUEUE_H_

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

#include <gcf/GCFTypedefs.h>

#include <dcl/DCLTypes.h>

#include "Request.h"

namespace dcl {

namespace comm {

class CreateCommandQueue: public Request {
public:
	CreateCommandQueue(
			dcl_object_id contextId,
			dcl_object_id deviceId,
			dcl_object_id commandQueueId,
			cl_command_queue_properties properties);
	CreateCommandQueue(const CreateCommandQueue&);

	dcl_object_id contextId(void) const;
	dcl_object_id deviceId(void) const;
	dcl_object_id commandQueueId(void) const;
	cl_command_queue_properties properties(void) const;

	DECLARE_SERIALIZABLE_PUBLIC(CreateCommandQueue, 100 + CREATE_COMMAND_QUEUE)

private:
	dcl_object_id _contextId;
	dcl_object_id _deviceId;
	dcl_object_id _commandQueueId;
	gcf_uint64 _properties;

	DECLARE_SERIALIZABLE_EVENT_PRIVATE(CreateCommandQueue)
};

} /* namespace comm */

} /* namespace dcl */

#endif /* CREATECOMMANDQUEUE_H_ */
