/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/**
 * @file    CreateProgramWithSource.h
 *
 * @date    07.06.2011
 * @author  Karsten Jeschkies
 */

#ifndef CREATEPROGRAMWITHSOURCE_H_
#define CREATEPROGRAMWITHSOURCE_H_

#include <cstddef>

#include <gcf/GCFTypedefs.h>

#include <dcl/DCLTypes.h>

#include "Request.h"

namespace dcl {

namespace comm {

class CreateProgramWithSource: public Request {
public:
	CreateProgramWithSource(
			const dcl_object_id program_id,
			const dcl_object_id context_id,
			const size_t length);
	CreateProgramWithSource(const CreateProgramWithSource&);
	virtual ~CreateProgramWithSource();

	dcl_object_id programId() const;
	dcl_object_id contextId() const;
	size_t length() const;

	DECLARE_SERIALIZABLE_PUBLIC(CreateProgramWithSource,
			100 + CREATE_PROGRAM_WITH_SOURCE);

private:
	dcl_object_id _programId;
	dcl_object_id _contextId;
	gcf_uint32 _length;

	DECLARE_SERIALIZABLE_EVENT_PRIVATE(CreateProgramWithSource)
};

} /* namespace comm */

} /* namespace dcl */

#endif /* CREATEPROGRAMWITHSOURCE_H_ */
