/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/*
 * EnqueueCopy.h
 *
 *  Created on: 22.06.2011
 *      Author: Philipp Kegel
 */

#ifndef ENQUEUECOPY_H_
#define ENQUEUECOPY_H_

#include <cstddef>
#include <vector>

#include <gcf/GCFTypedefs.h>

#include <dcl/DCLTypes.h>

#include "Request.h"

namespace dcl {

namespace comm {

class EnqueueCopyBuffer : public Request{
public:
	EnqueueCopyBuffer(
			dcl_object_id                      /* commandQueueId */,
			dcl_object_id                      /* commandId */,
			dcl_object_id                      /* srcBufferId */,
			dcl_object_id                      /* dstBufferId */,
			size_t                             /* srcOffset */,
			size_t                             /* dstOffset */,
			size_t                             /* cb */,
			const std::vector<dcl_object_id> *eventIdWaitList = 0,
			bool event = false);
	EnqueueCopyBuffer(const EnqueueCopyBuffer&);

    dcl_object_id commandQueueId() const;
    dcl_object_id commandId() const;
    dcl_object_id srcBufferId() const;
    dcl_object_id dstBufferId() const;
    size_t srcOffset() const;
    size_t dstOffset() const;
    size_t cb() const;
    const std::vector<dcl_object_id>& eventIdWaitList(void) const;
    bool event() const;

    DECLARE_SERIALIZABLE_PUBLIC(EnqueueCopyBuffer, 100 + ENQUEUE_COPY_BUFFER)

private:
    dcl_object_id _commandQueueId;
    dcl_object_id _commandId;
    dcl_object_id _srcBufferId;
    dcl_object_id _dstBufferId;
    gcf_uint32 _srcOffset;
    gcf_uint32 _dstOffset;
    gcf_uint32 _cb;
    std::vector<dcl_object_id> _eventIdWaitList;
    gcf_bool _event;

	DECLARE_SERIALIZABLE_EVENT_PRIVATE(EnqueueCopyBuffer)
};

} /* namespace comm */

} /* namespace dcl */

#endif /* ENQUEUECOPY_H_ */
