/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/*
 * @file    EnqueueMapMemObject.h
 *
 * @date    2012-08-08
 * @author  Philipp Kegel
 */

#ifndef ENQUEUEMAPMEMOBJECT_H_
#define ENQUEUEMAPMEMOBJECT_H_

#include <cstddef>
#include <vector>

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

#include <gcf/GCFTypedefs.h>

#include <dcl/DCLTypes.h>

#include "Request.h"

namespace dcl {

namespace comm {

class EnqueueMapBuffer : public Request{
public:
    EnqueueMapBuffer(
            dcl_object_id   /* commandQueueId */,
            dcl_object_id   /* commandId */,
            dcl_object_id   /* bufferId */,
            bool            /* blocking_map */,
            cl_map_flags    /* map_flags */,
            size_t          /* offset */,
            size_t          /* cb */,
            const std::vector<dcl_object_id> *eventIdWaitList = 0,
            bool event = 0);
    EnqueueMapBuffer(const EnqueueMapBuffer&);

    dcl_object_id commandQueueId() const;
    dcl_object_id commandId() const;
    dcl_object_id bufferId() const;
    bool blocking() const;
    cl_map_flags mapFlags() const;
    size_t offset() const;
    size_t cb() const;
    const std::vector<dcl_object_id>& eventIdWaitList(void) const;
    bool event() const;

    DECLARE_SERIALIZABLE_PUBLIC(EnqueueMapBuffer, 100 + ENQUEUE_MAP_BUFFER)

private:
    dcl_object_id _commandQueueId;
    dcl_object_id _commandId;
    dcl_object_id _bufferId;
    gcf_bool _blocking;
    gcf_uint64 _mapFlags;
    gcf_uint32 _offset;
    gcf_uint32 _cb;
    std::vector<dcl_object_id> _eventIdWaitList;
    gcf_bool _event;

    DECLARE_SERIALIZABLE_EVENT_PRIVATE(EnqueueMapBuffer)
};

/******************************************************************************/

class EnqueueUnmapMemObject : public Request{
public:
    EnqueueUnmapMemObject(
            dcl_object_id   /* commandQueueId */,
            dcl_object_id   /* commandId */,
            dcl_object_id   /* bufferId */,
            const std::vector<dcl_object_id> *eventIdWaitList = 0,
            bool event = 0);
    EnqueueUnmapMemObject(const EnqueueUnmapMemObject&);

    dcl_object_id commandQueueId() const;
    dcl_object_id commandId() const;
    dcl_object_id bufferId() const;
    const std::vector<dcl_object_id>& eventIdWaitList(void) const;
    bool event() const;

    DECLARE_SERIALIZABLE_PUBLIC(EnqueueUnmapMemObject, 100 + ENQUEUE_UNMAP_MEM_OBJECT)

private:
    dcl_object_id _commandQueueId;
    dcl_object_id _commandId;
    dcl_object_id _bufferId;
    std::vector<dcl_object_id> _eventIdWaitList;
    gcf_bool _event;

    DECLARE_SERIALIZABLE_EVENT_PRIVATE(EnqueueUnmapMemObject)
};

} /* namespace comm */

} /* namespace dcl */

#endif /* ENQUEUEMAPMEMOBJECT_H_ */
