/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/*
 * EnqueueWaitForEvents.h
 *
 *  Created on: 18.12.2011
 *      Author: Philipp Kegel
 */

#ifndef ENQUEUEWAITFOREVENTS_H_
#define ENQUEUEWAITFOREVENTS_H_

#include <vector>

#include <dcl/DCLTypes.h>

#include "Request.h"

namespace dcl {

namespace comm {

class EnqueueMarker: public Request {
public:
    EnqueueMarker(
            dcl_object_id   /* commandQueueId */,
            dcl_object_id   /* commandId */,
            const std::vector<dcl_object_id> *eventIdWaitList = 0,
            bool event = 0);
    EnqueueMarker(const EnqueueMarker&);

    dcl_object_id commandQueueId() const;
    dcl_object_id commandId() const;
    const std::vector<dcl_object_id>& eventIdWaitList(void) const;
    bool event() const;

    DECLARE_SERIALIZABLE_PUBLIC(EnqueueMarker, 100 + ENQUEUE_MARKER)

private:
    dcl_object_id _commandQueueId;
    dcl_object_id _commandId;
    std::vector<dcl_object_id> _eventIdWaitList;
    gcf_bool _event;

    DECLARE_SERIALIZABLE_EVENT_PRIVATE(EnqueueMarker)
};

/******************************************************************************/

class EnqueueWaitForEvents: public Request {
public:
    EnqueueWaitForEvents(
            dcl_object_id                       /* commandQueueId */,
            const std::vector<dcl_object_id>&   /* eventIdList */);
    EnqueueWaitForEvents(const EnqueueWaitForEvents&);

    dcl_object_id commandQueueId(void) const;
    const std::vector<dcl_object_id>& eventIdList(void) const;

    DECLARE_SERIALIZABLE_PUBLIC(EnqueueWaitForEvents, 100 + ENQUEUE_WAIT_FOR_EVENTS)

private:
    dcl_object_id _commandQueueId;
    std::vector<dcl_object_id> _eventIdList;

    DECLARE_SERIALIZABLE_EVENT_PRIVATE(EnqueueWaitForEvents)
};

/******************************************************************************/

class EnqueueBarrier: public Request {
public:
    EnqueueBarrier(
            dcl_object_id   /* commandQueueId */,
            dcl_object_id   /* commandId */,
            const std::vector<dcl_object_id> *eventIdWaitList = 0,
            bool event = 0);
    EnqueueBarrier(const EnqueueBarrier&);

    dcl_object_id commandQueueId() const;
    dcl_object_id commandId() const;
    const std::vector<dcl_object_id>& eventIdWaitList(void) const;
    bool event() const;

    DECLARE_SERIALIZABLE_PUBLIC(EnqueueBarrier, 100 + ENQUEUE_BARRIER)

private:
    dcl_object_id _commandQueueId;
    dcl_object_id _commandId;
    std::vector<dcl_object_id> _eventIdWaitList;
    gcf_bool _event;

    DECLARE_SERIALIZABLE_EVENT_PRIVATE(EnqueueBarrier)
};

} /* namespace comm */

} /* namespace dcl */

#endif /* ENQUEUEWAITFOREVENTS_H_ */
