/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/*
 * EnqueueWriteBuffer.h
 *
 *  Created on: 22.06.2011
 *      Author: luayabd
 */

#ifndef ENQUEUEWRITEBUFFER_H_
#define ENQUEUEWRITEBUFFER_H_

#include <cstddef>
#include <vector>

#include <gcf/GCFTypedefs.h>

#include <dcl/DCLTypes.h>

#include "Request.h"

namespace dcl {

namespace comm {

class EnqueueWriteBuffer : public Request{
public:
	EnqueueWriteBuffer(
			dcl_object_id /* commandQueueId */,
			dcl_object_id /* commandId */,
			dcl_object_id /* bufferId */,
			bool          /* blocking_write */,
			size_t        /* offset */,
			size_t        /* cb */,
			const std::vector<dcl_object_id> *eventWaitList = 0,
			bool event = false);
    EnqueueWriteBuffer(const EnqueueWriteBuffer&);

    dcl_object_id commandQueueId() const;
    dcl_object_id commandId() const;
    dcl_object_id bufferId() const;
    bool blocking() const;
    size_t offset() const;
    size_t cb() const;
    const std::vector<dcl_object_id>& eventIdWaitList(void) const;
    bool event() const;

    DECLARE_SERIALIZABLE_PUBLIC(EnqueueWriteBuffer, 100 + ENQUEUE_WRITE_BUFFER)

private:
    dcl_object_id _commandQueueId;
    dcl_object_id _commandId;
    dcl_object_id _bufferId;
    gcf_bool _blocking;
    gcf_uint32 _offset;
    gcf_uint32 _cb;
    std::vector<dcl_object_id> _eventIdWaitList;
    gcf_bool _event;

	DECLARE_SERIALIZABLE_EVENT_PRIVATE(EnqueueWriteBuffer)
};

} /* namespace comm */

} /* namespace dcl */

#endif /* ENQUEUEWRITEBUFFER_H_ */
