/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/*
 * ErrorResponse.h
 *
 *  Created on: 12.05.2011
 *      Author: karsten
 */

#ifndef ERRORRESPONSE_H_
#define ERRORRESPONSE_H_

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

#include "Response.h"

namespace dcl {

namespace comm {

/* forward declarations */
class Request;

/******************************************************************************/

class ErrorResponse: public Response {
public:
	ErrorResponse(
			const Request&,
			cl_int         /* errcode */);
	ErrorResponse(const ErrorResponse&);
    virtual ~ErrorResponse();

    cl_int errcode() const;

    /* TODO Replace message ID */
    DECLARE_SERIALIZABLE_PUBLIC(ErrorResponse, 299)

private:
	gcf_int32 _errcode;

	DECLARE_SERIALIZABLE_EVENT_PRIVATE(ErrorResponse)
};

} /* namespace comm */

} /* namespace dcl */

#endif /* ERRORRESPONSE_H_ */
