/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/**
 * @file EventSynchronizationMessage.h
 *
 * @date 2013-01-30
 * @author Sebastian Pribnow
 */

#ifndef EVENTSYNCHRONIZATIONMESSAGE_H_
#define EVENTSYNCHRONIZATIONMESSAGE_H_

#include <gcf/Serializable.h>

#include <dcl/DCLTypes.h>

namespace dcl {

namespace comm {

/**
 * @brief Request an update of memory objects, linked with this event.
 *
 * This message is sent by compute nodes to synchronize with an event, i.e. to
 * update to the changes associated with the specified event.
 */
class EventSynchronizationMessage: public gcf::Serializable {
public:
    EventSynchronizationMessage(
            const dcl_object_id eventId);
    EventSynchronizationMessage(
            const EventSynchronizationMessage& rhs);
    virtual ~EventSynchronizationMessage();

    dcl_object_id eventId() const;

    DECLARE_SERIALIZABLE_PUBLIC(EventSynchronizationMessage, 8802);

private:
    dcl_object_id _eventId;

    DECLARE_SERIALIZABLE_EVENT_PRIVATE(EventSynchronizationMessage);
};


} /* namespace comm */

} /* namespace dcl */


#endif /* EVENTSYNCHRONIZATIONMESSAGE_H_ */
