/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/**
 * @file    FlushFinishRequest.h
 *
 * @date    08.11.2011
 * @author  Philipp Kegel
 */

#ifndef FLUSHFINISHREQUEST_H_
#define FLUSHFINISHREQUEST_H_

#include <dcl/DCLTypes.h>

#include "Request.h"

namespace dcl {

namespace comm {

class FlushRequest: public Request {
public:
	FlushRequest(dcl_object_id /* commandQueueId */);
	FlushRequest(const FlushRequest&);
	virtual ~FlushRequest();

	dcl_object_id commandQueueId() const { return _commandQueueId; }

	DECLARE_SERIALIZABLE_PUBLIC(FlushRequest, 100 + FLUSH)

private:
	dcl_object_id _commandQueueId;

	DECLARE_SERIALIZABLE_EVENT_PRIVATE(FlushRequest)
};

/******************************************************************************/

class FinishRequest: public Request {
public:
	FinishRequest(dcl_object_id /* commandQueueId */);
	FinishRequest(const FinishRequest&);
	virtual ~FinishRequest();

	dcl_object_id commandQueueId() const { return _commandQueueId; }

	DECLARE_SERIALIZABLE_PUBLIC(FinishRequest, 100 + FINISH)

private:
	dcl_object_id _commandQueueId;

	DECLARE_SERIALIZABLE_EVENT_PRIVATE(FinishRequest)
};

} /* namespace comm */

} /* namespace dcl */

#endif /* FLUSHFINISHREQUEST_H_ */
