/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/*
 * @file GetKernelInfo.h
 *
 * @date    2011-06-22
 * @author  Karsten Jeschkies
 * @author  Philipp Kegel
 */

#ifndef GETKERNELINFO_H_
#define GETKERNELINFO_H_

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

#include <gcf/GCFTypedefs.h>

#include <dcl/DCLTypes.h>

#include "Request.h"

namespace dcl {

namespace comm {

class GetKernelInfo: public Request {
public:
    GetKernelInfo(
            dcl_object_id   /* kernelId */,
            cl_kernel_info  /* paramName */);
    GetKernelInfo(const GetKernelInfo&);
    virtual ~GetKernelInfo();

    dcl_object_id kernelId() const;
    cl_kernel_info paramName() const;

    DECLARE_SERIALIZABLE_PUBLIC(GetKernelInfo, 100 + GET_KERNEL_INFO)

private:
    dcl_object_id _kernelId;
    gcf_uint32 _paramName;

    DECLARE_SERIALIZABLE_EVENT_PRIVATE(GetKernelInfo)
};

/******************************************************************************/

class GetKernelWorkGroupInfo: public Request {
public:
    GetKernelWorkGroupInfo(
            dcl_object_id               /* kernelId */,
            dcl_object_id               /* deviceId */,
            cl_kernel_work_group_info   /* paramName */);
    GetKernelWorkGroupInfo(const GetKernelWorkGroupInfo&);
    virtual ~GetKernelWorkGroupInfo();

    dcl_object_id kernelId() const;
    dcl_object_id deviceId() const;
    cl_kernel_work_group_info paramName() const;

    DECLARE_SERIALIZABLE_PUBLIC(GetKernelWorkGroupInfo, 100 + GET_KERNEL_WORK_GROUP_INFO)

private:
    dcl_object_id _kernelId;
    dcl_object_id _deviceId;
    gcf_uint32 _paramName;

    DECLARE_SERIALIZABLE_EVENT_PRIVATE(GetKernelWorkGroupInfo)
};

} /* namespace comm */

} /* namespace dcl */

#endif /* GETKERNELINFO_H_ */
