/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/*
 * \file    DCLException.cpp
 *
 * \date    27.02.2012
 * \author  Philipp Kegel
 */

#include <string>

#include <dcl/DCLException.h>

namespace dcl {

DCLException::DCLException(const char *what) throw() {
	if (what) {
		/* a string must not be initialized with NULL */
		_what.assign(what);
	}
}

DCLException::DCLException(const std::string& what) throw () :
	_what(what) { }

DCLException::~DCLException() throw() { }

const char * DCLException::what() const throw() {
	return _what.c_str();
}

/******************************************************************************/

template<int N>
BasicException<N>::BasicException(const char *what) throw() :
	DCLException(what) { }

template<int N>
BasicException<N>::BasicException(const std::string& what) throw () :
	DCLException(what) { }

/* explicit instantiation */
template class BasicException<DCL_CONNECTION_ERROR>;
template class BasicException<DCL_IO_ERROR>;
template class BasicException<DCL_PROTOCOL_ERROR>;

/******************************************************************************/

InvalidArgument::InvalidArgument(int err, const char *what) throw () :
	DCLException(what), _err(err) { }

InvalidArgument::InvalidArgument(int err, const std::string& what) throw () :
	DCLException(what), _err(err) { }

InvalidArgument::~InvalidArgument() throw() { }

int InvalidArgument::err() const throw() {
	return _err;
}

} /* namespace dcl */
