/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/**
 * @file HostConnectionManagerImpl.h
 *
 * @date 26.10.2011
 * @author Philipp Kegel
 */

#ifndef HOSTCONNECTIONMANAGERIMPL_H_
#define HOSTCONNECTIONMANAGERIMPL_H_

#include <set>
#include <string>
#include <vector>

#include <gcf/Process.h>

#include <dcl/ComputeNode.h>
#include <dcl/ConnectionManager.h>

#include "ConnectionManagerImpl.h"

namespace dclgcf {

class ComputeNodeImpl;

/* forward declarations */
namespace comm {

class CLComputeNodeMessageDispatcher;
class CLResponseDispatcher;

} /* namespace comm */

/******************************************************************************/

class HostConnectionManagerImpl:
        public ConnectionManagerImpl,       // extend ConnectionManagerImpl
        public dcl::HostConnectionManager { // implement HostConnectionManager
public:
    HostConnectionManagerImpl();
    virtual ~HostConnectionManagerImpl();

    void start();
    void stop();

    /*
     * Host connection manager API
     */
    dcl::ComputeNode * createComputeNode(
            const std::string&  /* url */);
    void createComputeNodes(
            const std::vector<std::string>&     /* urls */,
            std::vector<dcl::ComputeNode *>&    /* computeNodes */);
    void destroyComputeNode(
            dcl::ComputeNode *  /* computeNode */);

    ComputeNodeImpl * createComputeNode(
            const gcf::Process& /* process */);
    /**
     * @brief Creates compute nodes from the given GCF processes.
     *
     * WARNING: Compute nodes returned by this method may not be connected such
     * that subsequent communication with these compute nodes will fail.
     */
    void createComputeNodes(
            const std::set<const gcf::Process *>&   /* processes */,
            std::vector<ComputeNodeImpl *>&         /* computeNodes */);
    void destroyComputeNode(
            ComputeNodeImpl *   /* computeNode */);

    void connectComputeNodes(
            const std::vector<ComputeNodeImpl *>&   /* computeNodes */);

protected:
    bool computeNodeConnectionLost(const gcf::Process&);

private:
    /* Connection manager must not be copied */
    HostConnectionManagerImpl(const HostConnectionManagerImpl&);
    HostConnectionManagerImpl& operator=(const HostConnectionManagerImpl&);

    comm::CLComputeNodeMessageDispatcher *_clMessageDispatcher;
    comm::CLResponseDispatcher *_clResponseDispatcher;
};

} /* namespace dclgcf */

#endif /* HOSTCONNECTIONMANAGERIMPL_H_ */
