/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/**
 * @file    HostImpl.cpp
 *
 * @date    26.10.2011
 * @author  Philipp Kegel
 */

#include <ostream>

#include <gcf/CommunicationManager.h>
#include <gcf/Process.h>

#include <dcl/CLObjectRegistry.h>
#include <dcl/CLCommandListener.h>

#include <dcl/util/Logger.h>

#include "HostImpl.h"
#include "ProcessImpl.h"

#include "comm/CLMessageDispatcher.h"
#include "comm/CLRequestDispatcher.h"

namespace dclgcf {

HostImpl::HostImpl(
		gcf::CommunicationManager& communicationManager,
		const gcf::Process& process,
		comm::CLRequestDispatcher *requestDispatcher) :
	ProcessImpl(communicationManager, process),
			_clRequestDispatcher(requestDispatcher) {
    _clMessageDispatcher = new comm::CLHostMessageDispatcher(this);

	_communicationManager.addMessageListener(*_clMessageDispatcher,
	        comm::CLMessageDispatcher::callbackPolicy());

	/* TODO Set connection status of host */

	dcl::util::Logger << dcl::util::Debug
			<< "Created host " << url() << std::endl;
}

HostImpl::~HostImpl() {
	_communicationManager.removeMessageListener(*_clMessageDispatcher);
	delete _clMessageDispatcher;
}

void HostImpl::setCLCommandListener(dcl::CLCommandListener *listener) {
	if (!_clRequestDispatcher) return;

	if (listener) {
		_clRequestDispatcher->addCLCommandListener(_process, *listener);
	} else {
		_clRequestDispatcher->removeCLCommandListener(_process);
	}
}

dcl::CLObjectRegistry& HostImpl::objectRegistry() {
    return _objectRegistry;
}

} /* namespace dclgcf */
