/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/**
 * @file    CreateKernel.cpp
 *
 * @date    Jun 8, 2011
 * @author  Tunc Taylan Turunc
 */

#include <string>

#include <gcf/GCFTypedefs.h>
#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>

#include <dcl/DCLTypes.h>
#include <dcl/comm/CreateKernel.h>
#include <dcl/comm/Request.h>

namespace dcl {

namespace comm {

CreateKernel::CreateKernel() :
	_kernelId(0), _programId(0) {
	initSerializable();
}

CreateKernel::CreateKernel(
		dcl_object_id kernelId,
		dcl_object_id programId,
		const std::string& kernelName) :
	_kernelId(kernelId), _programId(programId), _kernelName(kernelName) {
	initSerializable();
}

CreateKernel::CreateKernel(const CreateKernel& rhs) :
	Request(rhs), _kernelId(rhs._kernelId), _programId(rhs._programId),
			_kernelName(rhs._kernelName) {
	initSerializable();
}

dcl_object_id CreateKernel::kernelId() const {
	return _kernelId;
}

dcl_object_id CreateKernel::programId() const {
	return _programId;
}

const std::string& CreateKernel::kernelName() const {
	return _kernelName;
}

IMPLEMENT_SERIALIZABLE_DERIVED_EVENT(
		dcl::comm::CreateKernel,
		dcl::comm::Request,
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::CreateKernel, _kernelId);
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::CreateKernel, _programId);
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::CreateKernel, _kernelName);
)

} /* namespace comm */

} /* namespace dcl */
