/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/**
 * @file    CreateProgramWithSource.cpp
 *
 * @date    07.06.2011
 * @author  Karsten Jeschkies
 */

#include <cstddef>

#include <gcf/GCFTypedefs.h>
#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>

#include <dcl/comm/CreateProgramWithSource.h>
#include <dcl/comm/Request.h>

namespace dcl {

namespace comm {

CreateProgramWithSource::CreateProgramWithSource(
		dcl_object_id programId,
		dcl_object_id contextId,
		size_t length) :
	_programId(programId), _contextId(contextId),
	_length(static_cast<gcf_uint32>(length)) {
	initSerializable();
}

CreateProgramWithSource::CreateProgramWithSource(
		const CreateProgramWithSource& rhs) :
	Request(rhs), _programId(rhs._programId), _contextId(rhs._contextId),
			_length(rhs._length) {
	initSerializable();
}

CreateProgramWithSource::CreateProgramWithSource() :
	_programId(0), _contextId(0), _length(0) {
	initSerializable();
}

CreateProgramWithSource::~CreateProgramWithSource() { }

dcl_object_id CreateProgramWithSource::programId() const {
	return _programId;
}

dcl_object_id CreateProgramWithSource::contextId() const {
	return _contextId;
}

size_t CreateProgramWithSource::length() const {
	return static_cast<size_t>(_length);
}

IMPLEMENT_SERIALIZABLE_DERIVED_EVENT(
		dcl::comm::CreateProgramWithSource,
		dcl::comm::Request,
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::CreateProgramWithSource, _programId);
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::CreateProgramWithSource, _contextId);
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::CreateProgramWithSource, _length);
)

} /* namespace comm */

} /* namespace dcl */
