/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/**
 * @file    DeleteMemory.cpp
 *
 * @date    12.07.2012
 * @author  Philipp Kegel
 */

#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>

#include <dcl/DCLTypes.h>
#include <dcl/comm/DeleteMemory.h>
#include <dcl/comm/Request.h>

namespace dcl {

namespace comm {

//template<GCFTypeID_t TypeId>
//ReleaseObject<TypeId>::ReleaseObject() :
//	_objectId(0) {
//	initSerializable();
//}
//
//template<GCFTypeID_t TypeId>
//ReleaseObject<TypeId>::ReleaseObject(
//		dcl_object_id objectId) :
//	_objectId(objectId) {
//	initSerializable();
//}
//
//template<GCFTypeID_t TypeId>
//ReleaseObject<TypeId>::ReleaseObject(
//		const ReleaseObject& rhs) :
//	Request(rhs), _objectId(rhs._objectId) {
//	initSerializable();
//}
//
//template<GCFTypeID_t TypeId>
//ReleaseObject<TypeId>::~ReleaseObject() { }
//
//IMPLEMENT_SERIALIZABLE_DERIVED_EVENT(
//		dcl::comm::ReleaseObject,
//		dcl::Request,
//		ADD_ATTRIBUTE_DEFAULT(dcl::comm::ReleaseObject, _objectId);
//		)
//
///* Explicit template instantiations */
//template class ReleaseObject<Request::RELEASE_CONTEXT>;
//template class ReleaseObject<Request::RELEASE_COMMAND_QUEUE>;
//template class ReleaseObject<Request::RELEASE_MEM_OBJECT>;
//template class ReleaseObject<Request::RELEASE_PROGRAM>;
//template class ReleaseObject<Request::RELEASE_KERNEL>;


/******************************************************************************/

DeleteMemory::DeleteMemory() :
	_memObjectId(0) {
	initSerializable();
}

DeleteMemory::DeleteMemory(
		dcl_object_id memObjectId) :
	_memObjectId(memObjectId) {
	initSerializable();
}

DeleteMemory::DeleteMemory(
		const DeleteMemory& rhs) :
	Request(rhs), _memObjectId(rhs._memObjectId) {
	initSerializable();
}

DeleteMemory::~DeleteMemory() { }

IMPLEMENT_SERIALIZABLE_DERIVED_EVENT(
		dcl::comm::DeleteMemory,
		dcl::comm::Request,
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::DeleteMemory, _memObjectId);
		)

} /* namespace comm */

} /* namespace dcl */
