/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/*
 * DeviceIDsResponse.h
 *
 *  Created on: 18.05.2011
 *      Author: karsten
 */

#ifndef DEVICEIDSRESPONSE_H_
#define DEVICEIDSRESPONSE_H_

#include <vector>

#include <dcl/DCLTypes.h>

#include <dcl/comm/Response.h>
#include <dcl/comm/Request.h>

namespace dcl {

namespace comm {

/**
 * @brief Response message containing a list of device IDs
 */
class DeviceIDsResponse: public Response {
public:
	DeviceIDsResponse(
			const Request&,
			const std::vector<dcl_object_id>& /* deviceIds */);
	DeviceIDsResponse(const DeviceIDsResponse&);
	virtual ~DeviceIDsResponse();

	const std::vector<dcl_object_id>& deviceIds(void) const;

	DECLARE_SERIALIZABLE_PUBLIC(DeviceIDsResponse, 200 + Request::GET_DEVICE_IDS)

private:
	std::vector<dcl_object_id> _deviceIds;

	DECLARE_SERIALIZABLE_EVENT_PRIVATE(DeviceIDsResponse)
};

} /* namespace comm */

} /* namespace dcl */

#endif /* DEVICEIDSRESPONSE_H_ */
