/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/*
 * EnqueueWriteBuffer.cpp
 *
 *  Created on: 22.06.2011
 *      Author: luayabd
 */

#include <cstddef>
#include <vector>

#include <gcf/GCFTypedefs.h>
#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>

#include <dcl/DCLTypes.h>
#include <dcl/comm/EnqueueWriteBuffer.h>
#include <dcl/comm/Request.h>

namespace dcl {

namespace comm {

EnqueueWriteBuffer::EnqueueWriteBuffer() :
	_commandQueueId(0), _commandId(0), _bufferId(0), _event(false) {
	initSerializable();
}

EnqueueWriteBuffer::EnqueueWriteBuffer(
		dcl_object_id commandQueueId,
		dcl_object_id commandId,
		dcl_object_id bufferId,
		bool blocking_write,
		size_t offset,
		size_t cb,
		const std::vector<dcl_object_id> *eventIdWaitList,
		bool event) :
	_commandQueueId(commandQueueId), _commandId(commandId), _bufferId(bufferId),
			_blocking(static_cast<gcf_bool>(blocking_write)),
			_offset(static_cast<gcf_uint32>(offset)),
			_cb(static_cast<gcf_uint32>(cb)),
			_event(static_cast<gcf_bool>(event)) {
	initSerializable();

	if (eventIdWaitList) {
		_eventIdWaitList = *eventIdWaitList;
	}
}

EnqueueWriteBuffer::EnqueueWriteBuffer(const EnqueueWriteBuffer& rhs) :
	Request(rhs), _commandQueueId(rhs._commandQueueId),
			_commandId(rhs._commandId), _bufferId(rhs._bufferId),
			_blocking(rhs._blocking), _offset(rhs._offset), _cb(rhs._cb),
			_eventIdWaitList(rhs._eventIdWaitList),	_event(rhs._event) {
	initSerializable();
}

dcl_object_id EnqueueWriteBuffer::commandQueueId() const {
	return _commandQueueId;
}

dcl_object_id EnqueueWriteBuffer::commandId() const {
	return _commandId;
}

dcl_object_id EnqueueWriteBuffer::bufferId() const {
	return _bufferId;
}

bool EnqueueWriteBuffer::blocking() const {
	return static_cast<bool>(_blocking);
}

size_t EnqueueWriteBuffer::offset() const {
	return static_cast<size_t> (_offset);
}

size_t EnqueueWriteBuffer::cb() const {
	return static_cast<size_t> (_cb);
}

const std::vector<dcl_object_id>& EnqueueWriteBuffer::eventIdWaitList(void) const {
	return _eventIdWaitList;
}

bool EnqueueWriteBuffer::event() const {
	return static_cast<bool>(_event);
}


IMPLEMENT_SERIALIZABLE_DERIVED_EVENT(
		dcl::comm::EnqueueWriteBuffer,
		dcl::comm::Request,
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::EnqueueWriteBuffer, _commandQueueId);
        ADD_ATTRIBUTE_DEFAULT(dcl::comm::EnqueueWriteBuffer, _commandId);
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::EnqueueWriteBuffer, _bufferId);
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::EnqueueWriteBuffer, _blocking);
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::EnqueueWriteBuffer, _offset);
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::EnqueueWriteBuffer, _cb);
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::EnqueueWriteBuffer, _eventIdWaitList);
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::EnqueueWriteBuffer, _event);
	)

} /* namespace comm */

} /* namespace dcl */
