/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/*
 * ErrorResponse.cpp
 *
 *  Created on: 12.05.2011
 *      Author: karsten
 */

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

#include <gcf/GCFTypedefs.h>
#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>

#include <dcl/comm/ErrorResponse.h>
#include <dcl/comm/Response.h>

namespace dcl {

namespace comm {

/* forward declarations */
class Request;

/******************************************************************************/

ErrorResponse::ErrorResponse() :
	_errcode(CL_SUCCESS) {
	initSerializable();
}

ErrorResponse::ErrorResponse(
		const Request& request,
		cl_int errcode) :
	Response(request), _errcode(static_cast<gcf_int32>(errcode)) {

	initSerializable();
}

ErrorResponse::ErrorResponse(
		const ErrorResponse& rhs) :
	Response(rhs), _errcode(rhs._errcode) {
	initSerializable();
}

ErrorResponse::~ErrorResponse() { }

cl_int ErrorResponse::errcode() const {
	return static_cast<cl_int>(_errcode);
}

IMPLEMENT_SERIALIZABLE_DERIVED_EVENT(
		dcl::comm::ErrorResponse,
		dcl::comm::Response,
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::ErrorResponse, _errcode);
		)

} /* namespace comm */

} /* namespace dcl */
