/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/**
 * @file    FlushFinishRequest.cpp
 *
 * @date    08.11.2011
 * @author  Philipp Kegel
 */

#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>

#include <dcl/DCLTypes.h>
#include <dcl/comm/FlushFinishRequest.h>
#include <dcl/comm/Request.h>

namespace dcl {

namespace comm {

FlushRequest::FlushRequest() :
	_commandQueueId(0) {
	initSerializable();
}

FlushRequest::FlushRequest(
		dcl_object_id commandQueueId):
	_commandQueueId(commandQueueId) {
	initSerializable();
}

FlushRequest::FlushRequest(
		const FlushRequest& rhs) :
	Request(rhs), _commandQueueId(rhs._commandQueueId) {
	initSerializable();
}

FlushRequest::~FlushRequest() { }

IMPLEMENT_SERIALIZABLE_DERIVED_EVENT(
		dcl::comm::FlushRequest,
		dcl::comm::Request,
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::FlushRequest, _commandQueueId);
		)

/******************************************************************************/

FinishRequest::FinishRequest() :
	_commandQueueId(0) {
	initSerializable();
}

FinishRequest::FinishRequest(
		dcl_object_id commandQueueId):
	_commandQueueId(commandQueueId) {
	initSerializable();
}

FinishRequest::FinishRequest(
		const FinishRequest& rhs) :
	Request(rhs), _commandQueueId(rhs._commandQueueId) {
	initSerializable();
}

FinishRequest::~FinishRequest() { }

IMPLEMENT_SERIALIZABLE_DERIVED_EVENT(
		dcl::comm::FinishRequest,
		dcl::comm::Request,
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::FinishRequest, _commandQueueId);
		)

} /* namespace comm */

} /* namespace dcl */
