/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/*
 * GetDeviceIDs.cpp
 *
 *  Created on: 18.05.2011
 *      Author: karsten
 */

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

#include <gcf/GCFTypedefs.h>
#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>
#include <gcf/serialization/ByteBuffer.h>

#include <dcl/comm/Request.h>

#include "GetDeviceIDs.h"

namespace gcf {

template<>
size_t writeCustomSTR<dcl::comm::GetDeviceIDs> (
		dcl::comm::GetDeviceIDs *object,
		const gcf::GenericSerializeTemplateRecord& gstr,
		char *buffer) {
	assert(gstr.getType() == gcf::ST_Custom1);

	gcf::ByteBuffer buf(buffer, gstr.getLength());
	cl_device_type& deviceType =
			*gcf::GenericSerializeTemplateRecord::getAttribute<
					dcl::comm::GetDeviceIDs, cl_device_type,
					gcf::ST_Custom1>(object, gstr);

	buf << deviceType;

	return buf.getLength();
}

template<>
size_t readCustomSTR<dcl::comm::GetDeviceIDs> (
		dcl::comm::GetDeviceIDs *object,
		const gcf::GenericSerializeTemplateRecord& gstr,
		const char *buffer) {
	assert(gstr.getType() == gcf::ST_Custom1);

	gcf::ByteBuffer buf(buffer, gstr.getLength());
	/* The GenericSerializeTemplateRecord allows for accessing object
	 * attributes without using getter/setter methods. */
	cl_device_type& deviceType =
			*gcf::GenericSerializeTemplateRecord::getAttribute<
					dcl::comm::GetDeviceIDs, cl_device_type,
					gcf::ST_Custom1>(object, gstr);

	buf >> deviceType;

	return buf.getLength();
}

} /* namespace gcf */

/******************************************************************************/

namespace dcl {

namespace comm {

GetDeviceIDs::GetDeviceIDs() :
	_deviceType(CL_DEVICE_TYPE_ALL) {
	initSerializable();
}

GetDeviceIDs::GetDeviceIDs(
		cl_device_type deviceType) :
		_deviceType(deviceType) {
	initSerializable();
}

GetDeviceIDs::GetDeviceIDs(
		const GetDeviceIDs& rhs) :
	Request(rhs), _deviceType(rhs._deviceType) {
	initSerializable();
}

GetDeviceIDs::~GetDeviceIDs() { }

IMPLEMENT_SERIALIZABLE_DERIVED_EVENT(
		dcl::comm::GetDeviceIDs,
		dcl::comm::Request,
		ADD_ATTRIBUTE_CUSTOM(dcl::comm::GetDeviceIDs, _deviceType, gcf::ST_Custom1,
				cl_device_type, sizeof(cl_device_type), gcf::Reliable, gcf::Public);
)

} /* namespace comm */

} /* namespace dcl */
