/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/

/*
 * GetDeviceIDs.cpp
 *
 *  Created on: 18.05.2011
 *      Author: karsten
 */

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

#include <gcf/GCFTypedefs.h>
#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>

#include <dcl/DCLTypes.h>
#include <dcl/comm/Request.h>

#include "GetDeviceInfos.h"

namespace dcl {

namespace comm {

GetDeviceInfos::GetDeviceInfos():
	_device(0), _paramName(static_cast<gcf_uint32>(CL_DEVICE_NAME)) {
	initSerializable();
}

GetDeviceInfos::GetDeviceInfos(
		dcl_object_id device,
		cl_device_info paramName):
	_device(device), _paramName(static_cast<gcf_uint32>(paramName)) {
	initSerializable();
}

GetDeviceInfos::GetDeviceInfos(
		const GetDeviceInfos& rhs) :
	Request(rhs), _device(rhs._device), _paramName(rhs._paramName) {
	initSerializable();
}

GetDeviceInfos::~GetDeviceInfos() { }

IMPLEMENT_SERIALIZABLE_DERIVED_EVENT(
		dcl::comm::GetDeviceInfos,
		dcl::comm::Request,
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::GetDeviceInfos, _device);
		ADD_ATTRIBUTE_DEFAULT(dcl::comm::GetDeviceInfos, _paramName);
		)

} /* namespace comm */

} /* namespace dcl */
